/**
 *
 */
package de.intevation.gnv.action;

import java.util.Collection;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.w3c.dom.Node;

import de.intevation.gnv.action.sessionmodel.DiagrammOptions;
import de.intevation.gnv.action.sessionmodel.SessionModel;
import de.intevation.gnv.action.sessionmodel.SessionModelFactory;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClient;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClientFactory;
import de.intevation.gnv.artifactdatabase.objects.ArtifactDescription;
import de.intevation.gnv.artifactdatabase.objects.OutputMode;
import de.intevation.gnv.artifactdatabase.objects.OutputParameter;
import de.intevation.gnv.util.XMLUtils;
import de.intevation.gnv.util.XSLTransformer;

/**
 * @author Tim Englich <tim.englich@intevation.de>
 *
 */
public class ChangeOptionsAction extends ArtifactDatabaseActionBase {
    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(ChangeOptionsAction.class);
    /**
     * Constructor
     */
    public ChangeOptionsAction() {
        super();
    }

    @Override
    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        log.debug("NextArtifactStepAction.execute");
        try {
            SessionModel sm = SessionModelFactory.getInstance().getSessionModel(request);
            
            String target = request.getParameter("target");
            OutputMode outputMode = sm.getOutputMode(target);
            if (outputMode != null){
                
                Collection<OutputParameter> op = outputMode.getOutputParameters();
                if (op != null){
                    Iterator<OutputParameter> it = op.iterator();
                    DiagrammOptions diagrammOptions = new DiagrammOptions();
                    while (it.hasNext()){
                        OutputParameter parameter = it.next();
                        diagrammOptions.setValue(parameter.getName(), request.getParameter(parameter.getName()));
                        
                    }
                    sm.setDiagrammOptions(diagrammOptions);
                }
            }
            
            ArtifactDescription ad = sm.getArtifactDescription();
            ArtifactDatabaseClient adc = ArtifactDatabaseClientFactory.getInstance().getArtifactDatabaseClient();
            ArtifactDescription artifactDescription = adc.getCurrentStepDescription(sm.getSelectedArtifactFactory(), sm.getCurrentArtifact());
            
            Node currentUI = artifactDescription.getCurrentUI();
            XSLTransformer transformer = new XSLTransformer();
            String ui = transformer.transform(new XMLUtils().getNodeXPath(currentUI, "dynamic"), "UTF-8", request.getRealPath("WEB-INF/config/templates/describe-ui.xsl"));
            
            request.setAttribute("ui", ui);
            
            String staticUI = transformer.transform(new XMLUtils().getNodeXPath(currentUI, "static"), "UTF-8", request.getRealPath("WEB-INF/config/templates/describe-ui-static.xsl"));
            request.setAttribute("staticui", staticUI);
            request.setAttribute("furthertargets", (ad.getReachableStates() != null && !ad.getReachableStates().isEmpty()));
            
            if (ad.getReachableStates() != null && !ad.getReachableStates().isEmpty()){
            }else{
                request.setAttribute("diagramm", true);
            }
            return super.execute(mapping, form, request, response);
        } catch (Exception e) {
            log.error(e,e);
            request.setAttribute(CommunicationKeys.REQUEST_EXCEPTION_MESSAGE_ID, e.getMessage());
            return super.getExceptionForward(mapping);
        }
    }
    
    

}
